package com.taauav.admin.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.dto.TauvWaterSpectrumDTO;
import com.taauav.common.bean.Response;
import com.taauav.common.core.mps.BaseQuery;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.taauav.admin.mapper.TauvWaterSpectrumMapper;
import com.taauav.admin.entity.TauvWaterSpectrum;
import com.taauav.admin.service.ITauvWaterSpectrumService;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Calendar;
import java.util.UUID;

/**
 * 多光谱图Service业务层处理
 *
 * @author daixiantong
 * @date 20200514
 */
@Service
public class TauvWaterSpectrumServiceImpl extends BaseServiceImpl<TauvWaterSpectrumMapper, TauvWaterSpectrum> implements ITauvWaterSpectrumService {
    @Resource
    private Response response;
    @Value("${file.spectrumExtension}")
    private String spectrumExtension;
    @Value("${file.uploadFolder}")
    private String uploadFolder;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;

    /**
     * 自定义分页列表
     *
     * @param waterSpectrumDTO
     * @param query
     * @return
     */
    @Override
    public Response selectSpectrumPage(TauvWaterSpectrumDTO waterSpectrumDTO, BaseQuery query) {
        QueryWrapper<TauvWaterSpectrum> wrapper = new QueryWrapper<>();
        wrapper.eq("mark", 1);
        if (!StringUtils.isEmpty(waterSpectrumDTO.getInspectNo())) {
            wrapper.like("inspect_no", waterSpectrumDTO.getInspectNo().trim());
        }
        if (StringUtils.isNotNull(waterSpectrumDTO.getAreaId()) && waterSpectrumDTO.getAreaId().compareTo(BigInteger.ZERO) > 0) {
            wrapper.eq("area_id", waterSpectrumDTO.getAreaId());
        }
        if (!StringUtils.isEmpty(waterSpectrumDTO.getDriverName())) {
            wrapper.like("driver_name", waterSpectrumDTO.getDriverName().trim());
        }
        if (StringUtils.isNotNull(waterSpectrumDTO.getStartTime()) && StringUtils.isNotNull(waterSpectrumDTO.getEndTime())
                && StringUtils.isNotEmpty(waterSpectrumDTO.getStartTime().toString()) && StringUtils.isNotEmpty(waterSpectrumDTO.getEndTime().toString())) {
            wrapper.between("inspect_time", waterSpectrumDTO.getStartTime().toString(), waterSpectrumDTO.getEndTime().toString());
        }
        IPage page = new Page(query.getPage(), query.getPageSize());
        IPage<TauvWaterSpectrum> list = baseMapper.selectPage(page, wrapper);
        return response.success(list);
    }


    /**
     * 上传多光谱附件
     *
     * @param file
     * @return
     */
    @Override
    public Response uploadSpectrum(MultipartFile file) {
        if (file.isEmpty()) {
            return response.failure("请选择上传文件");
        }
        String fileName = file.getOriginalFilename();
        String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
        // 校验文件后缀
        String[] extensionList = spectrumExtension.split(",");
        boolean result = Arrays.asList(extensionList).contains(extension.toLowerCase());
        if (!result) {
            return response.failure("上传文件后缀不允许");
        }

        String newFileName = UUID.randomUUID().toString().replace("-", "").substring(0, 10) + "." + extension;
        Calendar calendar = Calendar.getInstance();
        String year = calendar.get(Calendar.YEAR) + "";
        String month = (calendar.get(Calendar.MONTH) + 1) + "";
        String day = calendar.get(Calendar.DATE) + "";
        String destPath = uploadFolder + "file/spectrum/" + year + month + day + "/";
        File destPathFile = new File(destPath);
        if (!destPathFile.isDirectory()) {
            destPathFile.mkdirs();
        }
        String fileUrl = destPath + newFileName;
        File dest = new File(fileUrl);
        try {
            file.transferTo(dest);
            String preUrl = fileUrl.replace(uploadFolder, uploadUrl);
            return response.success(preUrl);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("文件上传异常" + e.getMessage());
            return response.failure(e.getMessage());
        }
    }

    @Override
    public Response editSpectrum(TauvWaterSpectrum waterSpectrum) {
        // 处理图片路径
        if (StringUtils.isNotEmpty(waterSpectrum.getCodPic()) && waterSpectrum.getCodPic().contains(uploadFolder)) {
            waterSpectrum.setCodPic(waterSpectrum.getCodPic().replace(uploadFolder, uploadUrl));
        }
        if (StringUtils.isNotEmpty(waterSpectrum.getNh3nPic()) && waterSpectrum.getNh3nPic().contains(uploadFolder)) {
            waterSpectrum.setNh3nPic(waterSpectrum.getNh3nPic().replace(uploadFolder, uploadUrl));
        }
        if (StringUtils.isNotEmpty(waterSpectrum.getTpPic()) && waterSpectrum.getTpPic().contains(uploadFolder)) {
            waterSpectrum.setTpPic(waterSpectrum.getTpPic().replace(uploadFolder, uploadUrl));
        }
        if (StringUtils.isNotEmpty(waterSpectrum.getTnPic()) && waterSpectrum.getTnPic().contains(uploadFolder)) {
            waterSpectrum.setTnPic(waterSpectrum.getTnPic().replace(uploadFolder, uploadUrl));
        }
        if (StringUtils.isNotEmpty(waterSpectrum.getDoPic()) && waterSpectrum.getDoPic().contains(uploadFolder)) {
            waterSpectrum.setDoPic(waterSpectrum.getDoPic().replace(uploadFolder, uploadUrl));
        }
        if (StringUtils.isNotEmpty(waterSpectrum.getTubPic()) && waterSpectrum.getTubPic().contains(uploadFolder)) {
            waterSpectrum.setTubPic(waterSpectrum.getTubPic().replace(uploadFolder, uploadUrl));
        }
        Boolean res = updateById(waterSpectrum);
        if (!res) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    @Override
    public Response dropSpectrum(String ids) {
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.in("id", ids);
        wrapper.eq("mark", 1);
        wrapper.set("mark", 0);
        update(wrapper);
        return response.success("删除成功");
    }
}
