package com.taauav.front.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.query.UserAdminQuery;
import com.taauav.front.service.IUserAdminService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 外包人员表 前端控制器
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@RestController
@RequestMapping("/front/useradmin")
public class UserAdminController extends FrontBaseController {

    @Autowired
    private IUserAdminService userAdminService;

    @Autowired
    private Response response;

    private static final String controllerName = "UserAdmin";

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) UserAdminQuery query) {
        return userAdminService.getList(query);
    }

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody(required = true) @Valid UserAdmin entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userAdminService.add(entity);
    }

    /**
     * 获取用户详情
     *
     * @param id 用户ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return userAdminService.info(id);
    }

    /**
     * 编辑用户详情
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid UserAdmin entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userAdminService.edit(entity);
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return userAdminService.drop(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody UserAdmin entity) {
        return userAdminService.setStatus(entity);
    }

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/resetPwd")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.PASSWORD_PERMISSION)
    public Response resetPwd(@RequestBody UserAdmin entity) {
        return userAdminService.resetPwd(entity);
    }

    /**
     * 基础数据
     *
     * @return
     */
    @GetMapping("/getBaseData")
    public Response getBaseData() {
        return response.success(userAdminService.getBaseData());
    }

    /**
     * 获取河长列表
     *
     * @return
     */
    @PostMapping("/getDriverManagerList")
    public Response getDriverManagerList() {
        return userAdminService.getDriverManagerList();
    }

    /**
     * 获取人员列表
     *
     * @return
     */
    @PostMapping("/getUserAdminList")
    public Response getUserAdminList() {
        return userAdminService.getUserAdminList();
    }

}
